% Demo script reproducing the experiments reported in Table 1 of
% L. Azzari and A. Foi, "Variance Stabilization in Poisson Image Deblurring", Proc. IEEE ISBI 2017
%
%  v1.00, April 18, 2017
%
% http://www.cs.tut.fi/~foi/invansc/
%
%  L. Azzari and A. Foi - Tampere University of Technology - 2017 - All rights reserved.
% -----------------------------------------------------------------------------------------------
%
%   !! IMPORTANT !!
%   This demo requires:
%    * Exact unbiased inverse package "invansc"
%       http://www.cs.tut.fi/~foi/invansc/
%    * BM3D denoising filter
%       http://www.cs.tut.fi/~foi/GCF-BM3D/
% -----------------------------------------------------------------------------------------------
%

clear
%% number of noise realizations over which to average PSNR
N_reps=10; 

%% images to deblur
img_names = {'cameraman.tif',
             'moon_gray.png',
             'fluocells.tif',};
for ii=1:numel(img_names)
    y=im2double(imread(['images_ISBI',filesep,img_names{ii}]));
    Imgs{ii}=y;
end

%% peaks of the images
peaks = 255./ [1 5 10 50 100 200];

%% blur point-spread function (PSF)
v=fspecial('gaussian', 25, sqrt(3));  

%% compute table
PSNR_res = zeros(numel(img_names),numel(peaks));
for ind_img = 1:numel(img_names)
    y=Imgs{ind_img};
    for ind_peak = 1:numel(peaks)
        disp([img_names{ind_img} num2str(peaks(ind_peak))]);
        peak = peaks(ind_peak);
        y=y/max(y(:))*peak;
        y_blur = imfilter(y, v(end:-1:1,end:-1:1), 'circular'); % blurring
        PSNR_yhat = zeros(1,N_reps);
        for kk=1:N_reps
            %% generate Poisson obervation with different seed
            randn('seed',kk); rand('seed',kk);
            z=poissrnd(y_blur);
            %% deblur
            [yhat,~] = iterVSTpoissonDeb(z,v);
            %% PSNR of the restored image
            PSNR_yhat(kk) = 10*log10(peak^2/mean((y(:)-yhat(:)).^2));
        end
        PSNR_res(ind_img,ind_peak) = mean(PSNR_yhat);
    end
end
%%
tableString=num2str([peaks' PSNR_res'],4);
tableStringHead=[' Peak'];
for ind_img = 1:numel(img_names)
    while numel(tableStringHead)<(size(tableString,2)/(numel(img_names)+1)*ind_img)
        tableStringHead=[tableStringHead,' '];
    end
    tableStringHead=[tableStringHead,'',img_names{ind_img}(1:max(1,min(numel(img_names{ind_img}),floor(size(tableString,2)/(numel(img_names)+1)-1))))];
end
disp(repmat('-',[1 size(tableString,2)]))
disp(tableStringHead)
disp(repmat('-',[1 size(tableString,2)]))
disp(tableString)
